package com.gic.haoban.manage.service.service.impl;

import static org.slf4j.LoggerFactory.getLogger;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.JSONResponse;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.common.utils.StringUtil;
import com.gic.haoban.manage.api.dto.ContentMaterialDTO;
import com.gic.haoban.manage.api.dto.MaterialDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseQwDTO;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.dao.mapper.TabHaobanMaterialMapper;
import com.gic.haoban.manage.service.dao.mapper.TabMaterialContentMapper;
import com.gic.haoban.manage.service.entity.TabHaobanMaterial;
import com.gic.haoban.manage.service.entity.TabMaterialContent;
import com.gic.haoban.manage.service.service.MaterialService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.haoban.manage.service.util.CommonUtil;
import com.gic.wechat.api.enums.QywxMediaTypeEnum;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;
import com.gic.weimob.common.Md5Util;
import com.github.pagehelper.Page;

@Service
public class MaterialServiceImpl implements MaterialService {
	
    private static final Logger logger = getLogger(MaterialService.class);
    
	@Autowired
	private TabHaobanMaterialMapper mapper;
	@Autowired
	private TabMaterialContentMapper materialContentMapper ;
	@Autowired
	private WxEnterpriseService wxEnterpriseService ;
	@Autowired
	private QywxSuiteApiService qywxSuiteApiService ;
    @Autowired
    private Config config;

	@Override
	public List<MaterialDTO> listMaterialByCategoryId(String categoryId) {
		return EntityUtil.changeEntityListByJSON(MaterialDTO.class, mapper.listMaterialByCategoryId(categoryId));
	}

	@Override
	public void insertMaterial(MaterialDTO materialDTO) {
		materialDTO.setCreateTime(new Date());
		materialDTO.setUpdateTime(materialDTO.getCreateTime());
		materialDTO.setStatusFlag(1);
		materialDTO.setMaterialId(StringUtil.randomUUID());
		mapper.insert(EntityUtil.changeEntityByJSON(TabHaobanMaterial.class, materialDTO));
		
	}

	@Override
	public void batchInsertMaterial(List<MaterialDTO> materialList) {
		mapper.batchInsert(EntityUtil.changeEntityListByOrika(TabHaobanMaterial.class, materialList));
	}

	@Override
	public MaterialDTO selectMaterialById(String materialId) {
		return EntityUtil.changeEntityByJSON(MaterialDTO.class, mapper.selectByPrimaryKey(materialId));
	}

	@Override
    public List<MaterialDTO> listMaterialByIds(List<String> materialIds) {
        return EntityUtil.changeEntityListByOrika(MaterialDTO.class, mapper.listByids(materialIds));
    }

    @Override
    public void edit(MaterialDTO materialDTO) {
        materialDTO.setUpdateTime(new Date());
        mapper.updateByPrimaryKeySelective(EntityUtil.changeEntityByJSON(TabHaobanMaterial.class, materialDTO));
    }

	@Override
	public Page<TabHaobanMaterial> listMaterial(String wxEnterpriseId, String keyword, String categoryId, Integer materialType) {
		return mapper.listMaterial(wxEnterpriseId,keyword,categoryId,materialType);
	}

	@Override
	public Page<TabHaobanMaterial> listMaterial(String wxEnterpriseId, String keyword, List<String> categoryIds, Integer materialType) {
		List<TabHaobanMaterial> materials = mapper.listMaterialByCategoryIds(wxEnterpriseId, keyword, categoryIds, materialType);
		return (Page) materials;
	}

	@Override
	public MaterialDTO getMaterialByFromMaterialId(String fromMaterialId, String wxEnterpriseId) {
        TabHaobanMaterial tabHaobanMaterial = mapper.selectByfromMaterialId(fromMaterialId, wxEnterpriseId);
        return EntityUtil.changeEntityByJSON(MaterialDTO.class, tabHaobanMaterial);
    }

	@Override
	public List<MaterialDTO> listMaterialByFromMateralId(String fromMaterialId) {
		List<TabHaobanMaterial> list = mapper.listByfromMaterialId(fromMaterialId);
		return EntityUtil.changeEntityListByOrika(MaterialDTO.class, list);
	}

	@Override
	public void delmaterialByIds(List<String> ids) {
		mapper.delMaterialByIds(ids);
	}
	
	@Override
	public MaterialDTO getHasChangeMadieMaterialById(String materialId, int from) {
	    MaterialDTO materialDTO = null ;
        TabMaterialContent entity =null ;
        if(from == 1) {
            materialDTO = this.selectMaterialById(materialId);
        }
        if(from == 2) {
            entity =this.materialContentMapper.selectById(materialId) ;
            if(null != entity) {
                materialDTO = EntityUtil.changeEntityByJSON(MaterialDTO.class, entity) ;
            }
        }
        if (null == materialDTO) {
            return null;
        }
        Date lastUploadTime = materialDTO.getWxLastUploadTime();
        Date date = new Date();
        Date compareDate = DateUtils.addDays(date, -3);
        if (lastUploadTime != null && lastUploadTime.after(compareDate) && StringUtils.isNotBlank(materialDTO.getMediaId())) {
            return materialDTO;
        }
        Integer materialType = materialDTO.getMaterialType();
        String madiaId = materialDTO.getMediaId();
        //需要上传的类型
        List<Integer> typeList = Arrays.asList(2, 4, 5, 6);
        if (typeList.contains(materialType)) {
            madiaId = this.reUpdalodMetail(materialId,from);
        }
        materialDTO.setMediaId(madiaId);
        materialDTO.setWxLastUploadTime(date);
        return materialDTO;
        
	}
	
	  private String reUpdalodMetail(String materialId,int from) {
	      JSONResponse jp = this.reUpdalodMetailJson(materialId, from) ;
	      if(null == jp) {
	          return "";
	      }
	      if (jp.getErrorCode() == 0) {
	          return jp.getResult().toString() ;
	      }
	      return "";
	  }
	
	  public JSONResponse reUpdalodMetailJson(String materialId,int from) {
	      JSONResponse jp = new JSONResponse() ;
	      MaterialDTO old = null ;
          TabMaterialContent entity =null ;
          if(from == 1) {
              old = this.selectMaterialById(materialId);
          }
          if(from == 2) {
              entity =this.materialContentMapper.selectById(materialId) ;
              if(null != entity) {
                  old = EntityUtil.changeEntityByJSON(MaterialDTO.class, entity) ;
              }
              // 1群发 2朋友圈
              from = entity.getMediaType() ;
          }
          Integer type = old.getMaterialType();
          if (type == null) {
              return jp;
          }
          String wxEnterpriseId = old.getWxEnterpriseId();
          WxEnterpriseQwDTO qwDTO = this.wxEnterpriseService.getQwInfo(wxEnterpriseId) ;
          if (qwDTO == null) {
              logger.info("qwDTO is null");
              return jp;
          }
          String url = "";
          QywxMediaTypeEnum fileType = null;
          if (type == 2) {
              fileType = QywxMediaTypeEnum.IMAGE;
              url = old.getImgUrl();
          }
          if (type == 4) {
              fileType = QywxMediaTypeEnum.VIDEO;
              url = old.getLink();
          }
          if (type == 5) {
              fileType = QywxMediaTypeEnum.FILE;
              url = old.getLink();
          } else if (type == 6) {
              fileType = QywxMediaTypeEnum.IMAGE;
              url = old.getImgUrl();
          }
          if (fileType == null) {
              logger.error("materialId无类型,materialId:{}", materialId);
              return jp;
          }
          if(from == 2) {
              url = entity.getImgUrl() ;
          }
          String[] arr = null ;
          if(url.contains("?")) {
              arr = url.split("\\?")[0].split("/") ;
          }else {
             arr = url.split("/");
          }
          int count = arr.length;
          if(from==2) {
              // 朋友圈上传接口
              if (type == 2) {
                  if(url.contains("?")) {
                      
                  }else {
                      url+= "?imageView2/2/w/1440/h/1080" ;
                  }
              }
              logger.info("url={}",url);
              jp = qywxSuiteApiService.uploadAttachment(qwDTO.getThirdCorpid(), config.getWxSuiteid(), CommonUtil.getFileByte(url), arr[count - 1], fileType.getCode());
          }else {
              // 群发上传
              jp = qywxSuiteApiService.uploadMedia(qwDTO.getThirdCorpid(), config.getWxSuiteid(), CommonUtil.getFileByte(url), arr[count - 1], fileType.getCode());
          }
          if (jp.getErrorCode() == 0) {
              if(from==1) {
                  old.setWxLastUploadTime(new Date());
                  old.setMediaId(jp.getResult().toString());
                  this.edit(old);
              }
              if(from==2) {
                  entity.setWxLastUploadTime(new Date());
                  entity.setMediaId(jp.getResult().toString());
                  this.materialContentMapper.updateByPrimaryKey(entity) ;
              }
          }
          return jp;
	  }
	  
	  public List<String> getImageMediaId(String wxEnterpriseId, List<ContentMaterialDTO> imageList, int mediaType) {
	        List<String> materialIdList = new ArrayList<>();
	        imageList.forEach(dto -> {
	            String imageUrl = dto.getImgUrl();
	            int type = dto.getMaterialType();
	            String md5 = Md5Util.MD5(imageUrl);
	            TabMaterialContent tab = this.materialContentMapper.selectByUrl(wxEnterpriseId, md5, mediaType);
	            if (null != tab) {
	                materialIdList.add(tab.getMaterialId());
	            } else {
	                String id = StringUtil.randomUUID();
	                tab = new TabMaterialContent();
	                tab.setMaterialId(id);
	                tab.setMaterialType(type);
	                tab.setImgUrl(imageUrl);
	                tab.setUrlMd5(md5);
	                tab.setMediaType(mediaType);
	                tab.setWxEnterpriseId(wxEnterpriseId);
	                this.materialContentMapper.insert(tab);
	                materialIdList.add(id);
	            }
	        });
	        return materialIdList;
	    }
}
