package com.gic.haoban.manage.web.controller.target;

import com.alibaba.fastjson.JSONObject;
import com.gic.commons.web.qo.PageQo;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.haoban.manage.web.qo.target.QueryDataStatisticsCommonQO;
import com.gic.haoban.manage.web.utils.StoreAuthUtils;
import com.gic.haoban.manage.web.utils.target.DataTargetHttpUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 好办小程序-目标跟进
 * @Author guojx
 * @Date 2023/1/17 9:51
 */
@Controller
public class TargetController {
    @Autowired
    private StoreAuthUtils storeAuthUtils;

    /**
     * 目标跟进达成分析月份数据概览
     * @param qo
     * @return
     */
    @RequestMapping(value = "get-target-attainment-analysis-overview")
    @ResponseBody
    public RestResponse getTargetAttainmentAnalysisOverview(QueryDataStatisticsCommonQO qo) {
        JSONObject jsonObject = getCommon(qo, true);
        Map<String, Object> res = DataTargetHttpUtils.http(DataTargetHttpUtils.getParamNoPage(jsonObject).toJSONString(),
                qo.getTimeType() == 1 ? "data_mbr_target_attainment_analysis_year_overview" : "data_mbr_target_attainment_analysis_month_overview");
        return DataTargetHttpUtils.responseOfOne(res, null);
    }

    /**
     * 目标跟进核心目标达成概况
     * @return
     */
    @RequestMapping(value = "get-target-reach-overview")
    @ResponseBody
    public RestResponse getTargetReachOverview(QueryDataStatisticsCommonQO qo) {
        JSONObject jsonObject = getCommon(qo, false);
        Map<String, Object> res = DataTargetHttpUtils.http(DataTargetHttpUtils.getParamNoPage(jsonObject).toJSONString(),
                "data_mbr_target_reach_overview");
        return DataTargetHttpUtils.responseOfOne(res, null);
    }

    /**
     * 目标跟进核心指标趋势分月/累计
     * @param qo
     * @param chartType 1:分月 2：累计
     * @return
     */
    @RequestMapping(value = "get-target-chart")
    @ResponseBody
    public RestResponse getTargetChart(QueryDataStatisticsCommonQO qo, Integer chartType) {
        if (chartType == null) {
            chartType = 1;
        }
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("enterpriseId", qo.getEnterpriseId());
        Map<String, Object> res = DataTargetHttpUtils.http(DataTargetHttpUtils.getParamNoPage(jsonObject).toJSONString(),
                chartType == 1 ? "data_mbr_target_chart" : "data_mbr_target_cumulate_chart");
        return DataTargetHttpUtils.response(res);
    }

    /**
     * 目标跟进达成分析门店明细数据
     * @param qo
     * @param pageQo
     * @return
     */
    @RequestMapping(value = "get-target-analysis-store-detail")
    @ResponseBody
    public RestResponse getTargetAnalysisStoreDetail(QueryDataStatisticsCommonQO qo, PageQo pageQo) {
        JSONObject jsonObject = getCommon(qo, true);
        Map<String, Object> res = DataTargetHttpUtils.http(DataTargetHttpUtils.getParam(pageQo, jsonObject).toJSONString(),
                qo.getTimeType() == 1 ? "data_mbr_target_analysis_year_store_detail" : "data_mbr_target_analysis_month_store_detail");
        return DataTargetHttpUtils.responsePageData(res);
    }

    /**
     * 目标跟进门店会员分层明细数据
     * @param qo
     * @return
     */
    @RequestMapping(value = "get-target-store-mbr-hierarchy-detail")
    @ResponseBody
    public RestResponse getTargetStoreMbrHierarchyDetail(QueryDataStatisticsCommonQO qo) {
        JSONObject jsonObject = getCommon(qo, true);
        Map<String, Object> res = DataTargetHttpUtils.http(DataTargetHttpUtils.getParamNoPage(jsonObject).toJSONString(),
                "data_mbr_target_store_mbr_hierarchy_detail");
        return DataTargetHttpUtils.responseOfOne(res, null);
    }

    /**
     * 目标跟进门店会员631分层明细数据
     * @param qo
     * @return
     */
    @RequestMapping(value = "get-target-store-mbr-631-detail")
    @ResponseBody
    public RestResponse getTargetStoreMbr631Detail(QueryDataStatisticsCommonQO qo) {
        JSONObject jsonObject = getCommon(qo, true);
        Map<String, Object> res = DataTargetHttpUtils.http(DataTargetHttpUtils.getParamNoPage(jsonObject).toJSONString(),
                "data_mbr_target_store_mbr_631_detail");
        return DataTargetHttpUtils.response(res);
    }

    private JSONObject getCommon(QueryDataStatisticsCommonQO qo, boolean app) {
        JSONObject jsonObject = new JSONObject();
        if (app) {
            jsonObject.put("entId", qo.getEnterpriseId());
            jsonObject.put("monthDate", qo.getTime());
        } else {
            jsonObject.put("enterpriseId", qo.getEnterpriseId());
            if (qo.getTimeType() == 1) {
                jsonObject.put("year", qo.getTime());
            } else {
                jsonObject.put("month", qo.getTime());
            }
        }
        if (StringUtils.isBlank(qo.getStoreId())) {
            List<String> storeIdList = storeAuthUtils.queryClerkStoreIds(qo.getClerkId(), qo.getWxEnterpriseId());
            jsonObject.put("storeId", storeIdList.stream().collect(Collectors.joining(",")));
        } else {
            jsonObject.put("storeId", qo.getStoreId());
        }
        return jsonObject;
    }
}
